/* This is "heat13.tm", written to accompany Introduction to
   Scientific Programming by Joseph L. Zachary. */

/* This program is an implementation for the heat transfer problem
   described in Chapter 17 of the text, and it produces Mathematica
   animation output as described in Chapter 18 of the text.  It is
   based on "heat9.c", but has been modified so that the function
   "simulate" can be called by Mathematica via MathLink. */

/* This program can be translated into a C program by using "mprep". */


#define MAXSIZE 1001    /* The maximum number of segments that can be */
                        /* simulated.  This is all that must be modified */
                        /* to change the maximum size of the simulation. */



/* Returns the largest of "x", "y", and "z". */

double max (double x, double y, double z)
{
  if (x >= y && x >= z) {
    return(x);
  }
  if (y >= z) {
    return(y);
  }
  return(z);
}



/* Returns the smallest of "x", "y", and "z". */

double min (double x, double y, double z)
{
  if (x <= y && x <= z) {
    return(x);
  }
  if (y <= z) {
    return(y);
  }
  return(z);
}



/* This function initializes "cur" to reflect the segment
   boundary temperatures at time zero.  There are "segs" segments.
   The temperature of the "left" heat source, the "right" heat source,
   and the "rod" are given. */

void initRod (double cur[], int segs,
              double left, double right, double rod)
{
  int i;                /* Steps through "cur". */

  cur[0] = left;
  for (i = 1; i < segs; i = i+1) {
    cur[i] = rod;
  }
  cur[segs] = right;
}



/* This function modifies "cur" to reflect the passage of one tick
   of simulated time.  There are "segs" segments. */

void nextRod (double cur[], int segs)
{

  double nxt[MAXSIZE];  /* Used to temporarily hold the new temperatures */
                        /* of "cur" during the simulation. */
  
  int i;                /* Steps through "cur" and "nxt". */

  for (i = 1; i < segs; i = i+1) {
    nxt[i] = (cur[i-1] + cur[i+1]) / 2;
  }
  
  for (i = 1; i < segs; i = i+1) {
    cur[i] = nxt[i];
  }

}  



/* This function uses MathLink to communicate back to Mathematica a
   single frame of an animation showing the temperatures in "rod",
   which has the specified number of "segments".  The "length" of the
   rod in centimeters, as well as the "low" and "high" temperature
   bounds of the vertical axis are also given. */

void createFrame (double rod[], int segments,
		  double length, double low, double high)
{

  int i;            /* Steps through "rod" */


  /* Use MathLink function calls to create the Mathematica command
     that creates the frame. */

  MLPutFunction(stdlink, "ListPlot", 3);
  MLPutFunction(stdlink, "List", segments+1);

  for (i = 0; i <= segments; i = i+1) {
    MLPutFunction(stdlink, "List", 2);
    MLPutReal(stdlink, i*length/segments);
    MLPutReal(stdlink, rod[i]);
  }

  MLPutFunction(stdlink, "Rule", 2);
  MLPutSymbol(stdlink, "PlotJoined");
  MLPutSymbol(stdlink, "True");

  MLPutFunction(stdlink, "Rule", 2);
  MLPutSymbol(stdlink, "PlotRange");
  MLPutFunction(stdlink, "List", 2);
  MLPutReal(stdlink, low);
  MLPutReal(stdlink, high);

}



/* This template describes the interface of "simulate" to MathLink.  A
   template is required for every function that Mathematica is to
   call. */

:Begin:
:Function:      simulate
:Pattern:       simulate[diffusivity_Real, length_Real,
                         left_Real, right_Real, rod_Real,
                         duration_Real, segs_Integer]
:Arguments:     {diffusivity, length, left, right, rod,
                 duration, segs}
:ArgumentTypes: {Real, Real, Real, Real, Real,
                 Real, Integer}
:ReturnType:    Real
:End:


/* This function models heat transfer in a rod of uniform temperature
   that is insulated along its lengths and has a heat source applied
   to its ends.  The rod's thermal "diffusivity" in cm^2/sec and
   "length" in cm are given, as are the temperature of the "rod", the
   "left" heat source, and the "right" heat source in degrees Celsius.

   The function returns the approximate temperature of the center of
   the rod in degrees Celsius "duration" seconds after the heat sources
   are applied.  It also produces an animation of the heat transfer
   process.

   The implementation is based on a finite-element model with "segs"
   elements. */

double simulate (double diffusivity, double length,
                 double left, double right, double rod,
                 double duration, int segs)
{

  double delta,         /* Length in seconds of each simulated time step */

         clock,         /* Repeatedly counts simulated time from zero to */
                        /* one second before being reset. */

         low,           /* The lowest and highest of the right end, left */
	 high;		/* end, and inititial rod temperatures. */

  double cur[MAXSIZE];  /* Temperatures at the ends of each of the "segs" */
                        /* segments, where cur[0] is the temperature of */
                        /* the left heat source and cur[segs+1] is the */
                        /* temperature of the right heat source. */
  

  /* Initialize  "cur" and compute the length of the simulated
     time step. */

  initRod(cur, segs, left, right, rod);
  delta = (length*length/(segs*segs)) / (2*diffusivity);


  /* Determine the minimum and maximum temperatures in the simulation. */

  low = min(rod, left, right);
  high = max(rod, left, right);


  /* Begin a list containing all of the frames of the animation. */

  MLPutFunction(stdlink, "List", duration+1);


  /* Create a frame of the animation corresponding time zero. */

  createFrame(cur, segs, length, low, high);


  /* Set the one-second timer to zero. */

  clock = 0;


  /* Simulate the passage of time.  During each iteration, compute the
     new segment boundary temperatures in terms of the old segment
     boundary temperatures.  As each second passes, create a frame
     of the animation. */

  while (duration > 0) {

    nextRod(cur, segs);
    duration = duration - delta;

    clock = clock + delta;
    if (clock >= 1 - delta/2) {
      createFrame(cur, segs, length, low, high);
      clock = clock - 1;
    }
      
  }


  /* Return the temperature of the center segment boundary. */  

  return(cur[segs/2]);

}



/* This is the version of "main" that must be used if MathLink is to
   communicate with the program. */

int main (int argc, char *argv[]) {
  return(MLMain(argc, argv));
}
